@@ -97,6 +97,7 @@ class FieldRenderer(object):
     Subclasses should override `render` and `deserialize`.
     See their docstrings for details.
     """
+
     def __init__(self, field):
         self.field = field
         assert isinstance(self.field, AbstractField)
@@ -339,6 +340,8 @@ class FieldRenderer(object):
 
     def _deserialize(self, data):
         if isinstance(self.field.type, fatypes.Boolean):
+            if isinstance(data, bool):
+                 return data
             if data is not None:
                 if data.lower() in ['1', 't', 'true', 'yes']: return True
                 if data.lower() in ['0', 'f', 'false', 'no']: return False
@@ -357,6 +360,8 @@ class FieldRenderer(object):
                 return validators.float_(data, self)
 
         def _date(data):
+            if isinstance(data, datetime.date):
+                return data
             if data == 'YYYY-MM-DD' or data == '-MM-DD' or not data.strip():
                 return None
             try:
@@ -364,6 +369,8 @@ class FieldRenderer(object):
             except:
                 raise validators.ValidationError('Invalid date')
         def _time(data):
+            if isinstance(data, datetime.time):
+                return data
             if data == 'HH:MM:SS' or not data.strip():
                 return None
             try:
@@ -376,7 +383,16 @@ class FieldRenderer(object):
         if isinstance(self.field.type, fatypes.Time):
             return _time(data)
         if isinstance(self.field.type, fatypes.DateTime):
-            data_date, data_time = data.split(' ')
+            if isinstance(data, datetime.datetime):
+                return data
+            if 'Z' in data:
+                data = data.strip('Z')
+            if 'T' in data:
+                data_date, data_time = data.split('T')
+            elif ' ' in data:
+                data_date, data_time = data.split(' ')
+            else:
+                raise validators.ValidationError('Incomplete datetime: %s' % data)
             dt, tm = _date(data_date), _time(data_time)
             if dt is None and tm is None:
                 return None
@@ -448,7 +464,7 @@ class PasswordFieldRenderer(TextFieldRenderer):
     def render(self, **kwargs):
         return h.password_field(self.name, value=self.value, maxlength=self.length, **kwargs)
     def render_readonly(self):
-        return '*'*6
+        return '*' * 6
 
 class TextAreaFieldRenderer(FieldRenderer):
     """render a field as a textarea"""
@@ -556,7 +572,7 @@ class DateFieldRenderer(FieldRenderer):
         is_date_type = isinstance(value, (datetime.datetime, datetime.date, datetime.time))
         values = []
         for key, default in (('month', 'MM'), ('day', 'DD')):
-            name = self.name+'__'+key
+            name = self.name + '__' + key
             v = default
             if data is not None and name in data:
                 v = data[name]
@@ -603,7 +619,7 @@ class TimeFieldRenderer(FieldRenderer):
         is_time_type = isinstance(value, (datetime.datetime, datetime.date, datetime.time))
         values = []
         for key, default in (('hour', 'HH'), ('minute', 'MM'), ('second', 'SS')):
-            name = self.name+'__'+key
+            name = self.name + '__' + key
             v = default
             if data is not None and name in data:
                 v = data[name]
@@ -634,6 +650,102 @@ class DateTimeFieldRenderer(DateFieldRenderer, TimeFieldRenderer):
         return DateFieldRenderer._serialized_value(self) + ' ' + TimeFieldRenderer._serialized_value(self)
 
 
+class EmailFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 email input field
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='email', **kwargs)
+
+
+class UrlFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 url input field
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='url', **kwargs)
+
+
+class NumberFieldRenderer(IntegerFieldRenderer):
+    '''
+    Render a HTML5 number input field
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='number', **kwargs)
+
+
+class RangeFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 range input field
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='range', **kwargs)
+
+
+class HTML5DateFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 date input field
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='date', **kwargs)
+
+class HTML5DateTimeFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 datetime input field
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='datetime', **kwargs)
+
+class LocalDateTimeFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 datetime-local input field.
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='datetime-local', **kwargs)
+
+
+class MonthFieldRender(FieldRenderer):
+    '''
+    Render a HTML5 month input field.
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='month', **kwargs)
+
+
+class WeekFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 week input field.
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='week', **kwargs)
+
+
+class HTML5TimeFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 time input field.
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='time', **kwargs)
+
+class ColorFieldRenderer(FieldRenderer):
+    '''
+    Render a HTML5 color input field.
+    '''
+
+    def render(self, **kwargs):
+        return h.text_field(self.name, value=self.value, type='color', **kwargs)
+
+
 def _extract_options(options):
     if isinstance(options, dict):
         options = options.items()
@@ -815,9 +927,9 @@ def _pk(instance):
 # 2.6 provides ast.literal_eval, but requiring 2.6 is a bit of a stretch for now.
 import compiler
 class _SafeEval(object):
-    def visit(self, node,**kw):
+    def visit(self, node, **kw):
         cls = node.__class__
-        meth = getattr(self,'visit'+cls.__name__,self.default)
+        meth = getattr(self, 'visit' + cls.__name__, self.default)
         return meth(node, **kw)
 
     def default(self, node, **kw):
@@ -833,10 +945,10 @@ class _SafeEval(object):
     def visitConst(self, node, **kw):
         return node.value
 
-    def visitTuple(self,node, **kw):
+    def visitTuple(self, node, **kw):
         return tuple(self.visit(i) for i in node.nodes)
 
-    def visitList(self,node, **kw):
+    def visitList(self, node, **kw):
         return [self.visit(i) for i in node.nodes]
 
 def _simple_eval(source):
@@ -1119,6 +1231,8 @@ class AbstractField(object):
                 else:
                     renderer = HiddenFieldRenderer
                 self._renderer = renderer
+            elif attr in 'attrs':
+                self.render_opts.update(value)
             elif attr in mapping:
                 attr = mapping.get(attr)
                 setattr(self, attr, value)
@@ -1227,6 +1341,10 @@ class AbstractField(object):
         else:
             html_options['class_'] = self.is_required() and 'field_req' or 'field_opt'
         return h.content_tag('label', self.label(), **html_options)
+    def attrs(self, **kwargs):
+        """update ``render_opts``"""
+        self.render_opts.update(kwargs)
+        return self._modified(render_opts=self.render_opts)
     def readonly(self, value=True):
         """Render the field readonly."""
         return self._modified(_readonly=True)
@@ -1298,6 +1416,120 @@ class AbstractField(object):
         """
         return deepcopy(self.parent._fields[self.name])
 
+    #==========================================================================
+    # HTML5 specific input types
+    #==========================================================================
+
+    def date(self):
+        '''
+        Render the field as a HTML5 date input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['date']
+        return field
+
+    def datetime(self):
+        '''
+        Render the field as a HTML5 datetime input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['datetime']
+        return field
+
+    def datetime_local(self):
+        '''
+        Render the field as a HTML5 datetime-local input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['date']
+        return field
+
+    def month(self):
+        '''
+        Render the field as a HTML5 month input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['month']
+        return field
+
+    def week(self):
+        '''
+        Render the field as a HTML5 week input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['week']
+        return field
+
+    def time(self):
+        '''
+        Render the field as a HTML5 time input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['time']
+        return field
+
+    def color(self):
+        '''
+        Render the field as a HTML5 color input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['color']
+        return field
+
+    def range(self, min_=None, max_=None, step=None, value=None):
+        '''
+        Render the field as a HTML5 range input type, starting at `min_`,
+        ending at `max_`, with legal increments every `step` distance.  The
+        default is set by `value`.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['range']
+        field.render_opts = {}
+        if min_:
+            field.render_opts["min"] = min_
+        if max_:
+            field.render_opts["max"] = max_
+        if step:
+            field.render_opts["step"] = step
+        if value:
+            field.render_opts["value"] = value
+        return field
+
+    def number(self, min_=None, max_=None, step=None, value=None):
+        '''
+        Render the field as a HTML5 number input type, starting at `min_`,
+        ending at `max_`, with legal increments every `step` distance.  The
+        default is set by `value`.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['number']
+        field.render_opts = {}
+        if min_:
+            field.render_opts["min"] = min_
+        if max_:
+            field.render_opts["max"] = max_
+        if step:
+            field.render_opts["step"] = step
+        if value:
+            field.render_opts["value"] = value
+        return field
+
+    def url(self):
+        '''
+        Render the field as a HTML5 url input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['url']
+        return field
+
+    def email(self):
+        '''
+        Render the field as a HTML5 email input type.
+        '''
+        field = deepcopy(self)
+        field._renderer = lambda f: f.parent.default_renderers['email']
+        return field
+
     def _get_renderer(self):
         for t in self.parent.default_renderers:
             if not isinstance(t, basestring) and type(self.type) is t:
